package org.LiveGraph.gui;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Toolkit;

import javax.swing.Box;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JScrollBar;
import javax.swing.JTextArea;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.WindowConstants;
import javax.swing.JScrollPane;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.filechooser.FileFilter;

import org.LiveGraph.LiveGraph;
import org.LiveGraph.settings.DataFileSettings;
import org.LiveGraph.settings.DataSeriesSettings;
import org.LiveGraph.settings.ErrorWhileSettingHasChangedProcessingException;
import org.LiveGraph.settings.GraphSettings;


import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * This is the main window of the application.
 * 
 * <p style="font-size:smaller;">This product includes software developed by the
 *    <strong>LiveGraph</strong> project and its contributors.<br />
 *    (<a href="http://www.live-graph.org" target="_blank">http://www.live-graph.org</a>)<br />
 *    Copyright (c) 2007 G. Paperin.<br />
 *    All rights reserved.
 * </p>
 * <p style="font-size:smaller;">File: MainWindow.java</p> 
 * <p style="font-size:smaller;">Redistribution and use in source and binary forms, with or
 *    without modification, are permitted provided that the following terms and conditions are met:
 * </p>
 * <p style="font-size:smaller;">1. Redistributions of source code must retain the above
 *    acknowledgement of the LiveGraph project and its web-site, the above copyright notice,
 *    this list of conditions and the following disclaimer.<br />
 *    2. Redistributions in binary form must reproduce the above acknowledgement of the
 *    LiveGraph project and its web-site, the above copyright notice, this list of conditions
 *    and the following disclaimer in the documentation and/or other materials provided with
 *    the distribution.<br />
 *    3. All advertising materials mentioning features or use of this software or any derived
 *    software must display the following acknowledgement:<br />
 *    <em>This product includes software developed by the LiveGraph project and its
 *    contributors.<br />(http://www.live-graph.org)</em><br />
 *    4. All advertising materials distributed in form of HTML pages or any other technology
 *    permitting active hyper-links that mention features or use of this software or any
 *    derived software must display the acknowledgment specified in condition 3 of this
 *    agreement, and in addition, include a visible and working hyper-link to the LiveGraph
 *    homepage (http://www.live-graph.org).
 * </p>
 * <p style="font-size:smaller;">THIS SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY
 *    OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND  NONINFRINGEMENT. IN NO EVENT SHALL
 *    THE AUTHORS, CONTRIBUTORS OR COPYRIGHT  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING  FROM, OUT OF OR
 *    IN CONNECTION WITH THE SOFTWARE OR THE USE OR  OTHER DEALINGS IN THE SOFTWARE.
 * </p>
 * 
 * @author Greg Paperin (<a href="http://www.paperin.org" target="_blank">http://www.paperin.org</a>)
 * @version {@value org.LiveGraph.LiveGraph#version}
 */
public class MainWindow extends JFrame {

private static final DateFormat logTimestampFormat = new SimpleDateFormat("HH:mm:ss");

private JTextArea messageArea = null;
private JCheckBoxMenuItem dataFileSettingsDisplayMenuItem = null;
private JCheckBoxMenuItem graphSettingsDisplayMenuItem = null;
private JCheckBoxMenuItem seriesSettingsDisplayMenuItem = null;
private JCheckBoxMenuItem plotDisplayMenuItem = null;

/**
 * This is the default constructor. 
 */
public MainWindow() {
	super();
	initialize();
}


/**
 * This method initializes the main window.
 */
private void initialize() {
	
	// Main window size and position:
	final int WIN_WIDTH = 600;
	final int WIN_HEIGHT = 200;
	Dimension scr = Toolkit.getDefaultToolkit().getScreenSize();
	this.setBounds((scr.width - WIN_WIDTH) / 2, (scr.height - WIN_HEIGHT) / 2, WIN_WIDTH, WIN_HEIGHT);
	
	// Other main settings:
	final MainWindow MAIN_WIN = this;
	this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);	
	this.setTitle("LiveGraph");	
	getContentPane().setLayout(new BorderLayout());
	
	// CLosing handler:
	this.addWindowListener(new WindowAdapter() {
		@Override public void windowClosing(WindowEvent e) {
			LiveGraph.application().disposeGUIAndExit();
		}
	});
	
	// Message area in the centre:
	JScrollPane centreScrollPane = new JScrollPane();	
	getContentPane().add(centreScrollPane, BorderLayout.CENTER);
	messageArea = new JTextArea();
	messageArea.setEditable(false);
	messageArea.setFont(new Font("Courier New", Font.PLAIN, 12));
	centreScrollPane.setViewportView(messageArea);
	
	// Menu:
	JMenuBar menuBar = new JMenuBar();
	this.setJMenuBar(menuBar);
	JMenu menu = null;
	JMenuItem mItem = null;
	
	menu = new JMenu("Data file");
	final JFileChooser dataFileSettingsFileDlg = new JFileChooser();
	dataFileSettingsFileDlg.setFileFilter(new FileFilter() {
		@Override public boolean accept(File f) {
        	if (null == f) return false;
        	if (f.isDirectory()) return true;
        	int p = f.getName().lastIndexOf(".");        
        	return p < 0
        			? false
        			: f.getName().substring(p).equalsIgnoreCase(DataFileSettings.preferredFileExtension);
	    }
		@Override public String getDescription() {
        	return "LiveGraph data file settings (*" + DataFileSettings.preferredFileExtension + ")";
        }
	});	
	dataFileSettingsFileDlg.setCurrentDirectory(new File(System.getProperty("user.dir")));
	mItem = new JMenuItem("Load data file settings...");
	mItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (JFileChooser.APPROVE_OPTION != dataFileSettingsFileDlg.showOpenDialog(MAIN_WIN))
				return;
			File selFile = dataFileSettingsFileDlg.getSelectedFile();
			if (!selFile.exists())
				return;
			
			try {				
				if (LiveGraph.application().getDataFileSettings().load(selFile.getAbsolutePath()))
					LiveGraph.application().logSuccessLn("Data file settings loaded from " + selFile.getName() + ".");
				else
					LiveGraph.application().logErrorLn("Error while loading data file settings from " + selFile.getName() + ".");
				
			} catch (ErrorWhileSettingHasChangedProcessingException ex) {
				logErrorLn("There was a problem while loading data file settings: \n"
						 + "    " + (null != ex.getCause() ? ex.getCause().getMessage() : ex.getMessage()) + ".");
			}
		}
	});
	menu.add(mItem);
	mItem = new JMenuItem("Save data file settings...");
	menu.add(mItem);
	mItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (JFileChooser.APPROVE_OPTION != dataFileSettingsFileDlg.showSaveDialog(MAIN_WIN))
				return;
			File selFile = dataFileSettingsFileDlg.getSelectedFile();
			if (!selFile.getParentFile().exists())
				return;
			if (!selFile.getName().contains("."))
				selFile = new File(selFile.getAbsolutePath() + DataFileSettings.preferredFileExtension);  
			if (LiveGraph.application().getDataFileSettings().save(selFile.getAbsolutePath()))
				LiveGraph.application().logSuccessLn("Data file settings saved to " + selFile.getName() + ".");
			else
				LiveGraph.application().logErrorLn("Error while saving data file settings to " + selFile.getName() + ".");			
		}
	});
	menu.addSeparator();
	dataFileSettingsDisplayMenuItem = new JCheckBoxMenuItem("Display data file settings");
	dataFileSettingsDisplayMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().setDisplayDataFileSettingsWindow(dataFileSettingsDisplayMenuItem.getState());
		}
	});
	menu.add(dataFileSettingsDisplayMenuItem);	
	menuBar.add(menu);
	
	menu = new JMenu("Graph settings");
	final JFileChooser graphSettingsFileDlg = new JFileChooser();
	graphSettingsFileDlg.setFileFilter(new FileFilter() {
		@Override public boolean accept(File f) {
        	if (null == f) return false;
        	if (f.isDirectory()) return true;
        	int p = f.getName().lastIndexOf(".");        
        	return p < 0
        			? false
        			: f.getName().substring(p).equalsIgnoreCase(GraphSettings.preferredFileExtension);
	    }
		@Override public String getDescription() {
        	return "LiveGraph graph settings (*" + GraphSettings.preferredFileExtension + ")";
        }
	});	
	graphSettingsFileDlg.setCurrentDirectory(new File(System.getProperty("user.dir")));
	mItem = new JMenuItem("Load graph settings...");
	mItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (JFileChooser.APPROVE_OPTION != graphSettingsFileDlg.showOpenDialog(MAIN_WIN))
				return;
			File selFile = graphSettingsFileDlg.getSelectedFile();
			if (!selFile.exists())
				return;
			
			try {				
				if (LiveGraph.application().getGraphSettings().load(selFile.getAbsolutePath()))
					LiveGraph.application().logSuccessLn("Graph settings loaded from " + selFile.getName() + ".");
				else
					LiveGraph.application().logErrorLn("Error while loading graph settings from " + selFile.getName() + ".");
				
			} catch (ErrorWhileSettingHasChangedProcessingException ex) {
				logErrorLn("There was a problem while loading graph settings: \n"
						 + "    " + (null != ex.getCause() ? ex.getCause().getMessage() : ex.getMessage()) + ".");
			}
		}
	});
	menu.add(mItem);
	mItem = new JMenuItem("Save graph settings...");
	mItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (JFileChooser.APPROVE_OPTION != graphSettingsFileDlg.showSaveDialog(MAIN_WIN))
				return;
			File selFile = graphSettingsFileDlg.getSelectedFile();
			if (!selFile.getParentFile().exists())
				return;
			if (!selFile.getName().contains("."))
				selFile = new File(selFile.getAbsolutePath() + GraphSettings.preferredFileExtension);  
			if (LiveGraph.application().getGraphSettings().save(selFile.getAbsolutePath()))
				LiveGraph.application().logSuccessLn("Graph settings saved to " + selFile.getName() + ".");
			else
				LiveGraph.application().logErrorLn("Error while saving graph settings to " + selFile.getName() + ".");			
		}
	});
	menu.add(mItem);
	menu.addSeparator();
	graphSettingsDisplayMenuItem = new JCheckBoxMenuItem("Display graph settings");
	graphSettingsDisplayMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().setDisplayGraphSettingsWindow(graphSettingsDisplayMenuItem.getState());
		}
	});
	menu.add(graphSettingsDisplayMenuItem);	
	menuBar.add(menu);
	
	menu = new JMenu("Data series");
	final JFileChooser seriesSettingsFileDlg = new JFileChooser();
	seriesSettingsFileDlg.setFileFilter(new FileFilter() {
		@Override public boolean accept(File f) {
        	if (null == f) return false;
        	if (f.isDirectory()) return true;
        	int p = f.getName().lastIndexOf(".");        
        	return p < 0
        			? false
        			: f.getName().substring(p).equalsIgnoreCase(DataSeriesSettings.preferredFileExtension);
	    }
		@Override public String getDescription() {
        	return "LiveGraph data series settings (*" + DataSeriesSettings.preferredFileExtension + ")";
        }
	});
	seriesSettingsFileDlg.setCurrentDirectory(new File(System.getProperty("user.dir")));
	mItem = new JMenuItem("Load data series settings...");
	mItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (JFileChooser.APPROVE_OPTION != seriesSettingsFileDlg.showOpenDialog(MAIN_WIN))
				return;
			File selFile = seriesSettingsFileDlg.getSelectedFile();
			if (!selFile.exists())
				return;
			
			try {
				if (LiveGraph.application().getDataSeriesSettings().load(selFile.getAbsolutePath()))
					LiveGraph.application().logSuccessLn("Data series settings loaded from " + selFile.getName() + ".");
				else
					LiveGraph.application().logErrorLn("Error while loading data series settings from " + selFile.getName() + ".");
			
			} catch (ErrorWhileSettingHasChangedProcessingException ex) {
				logErrorLn("There was a problem while loading data series settings: \n"
						 + "    " + (null != ex.getCause() ? ex.getCause().getMessage() : ex.getMessage()) + ".");
			}
		}
	});
	menu.add(mItem);
	mItem = new JMenuItem("Save data series settings...");
	mItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (JFileChooser.APPROVE_OPTION != seriesSettingsFileDlg.showSaveDialog(MAIN_WIN))
				return;
			File selFile = seriesSettingsFileDlg.getSelectedFile();
			if (!selFile.getParentFile().exists())
				return;
			if (!selFile.getName().contains("."))
				selFile = new File(selFile.getAbsolutePath() + DataSeriesSettings.preferredFileExtension);  
			if (LiveGraph.application().getDataSeriesSettings().save(selFile.getAbsolutePath()))
				LiveGraph.application().logSuccessLn("Data series settings saved to " + selFile.getName() + ".");
			else
				LiveGraph.application().logErrorLn("Error while saving data series settings to " + selFile.getName() + ".");			
		}
	});
	menu.add(mItem);
	menu.addSeparator();
	seriesSettingsDisplayMenuItem = new JCheckBoxMenuItem("Display data series settings");	
	seriesSettingsDisplayMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().setDisplaySeriesSettingsWindow(seriesSettingsDisplayMenuItem.getState());
		}
	});
	menu.add(seriesSettingsDisplayMenuItem);	
	menuBar.add(menu);
	
	menu = new JMenu("Plot");
	mItem = new JMenuItem("Export graph to image...");
	mItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().getGraphExporter().exportGraph();			
		}
	});
	menu.add(mItem);
	menu.addSeparator();
	plotDisplayMenuItem = new JCheckBoxMenuItem("Display plot window");
	plotDisplayMenuItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			LiveGraph.application().setDisplayPlotWindow(plotDisplayMenuItem.getState());
		}
	});
	menu.add(plotDisplayMenuItem);	
	menuBar.add(menu);

	menuBar.add(Box.createHorizontalGlue());
	menu = new JMenu("Help");	
	mItem = new JMenuItem("Support...");
	mItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			JOptionPane.showMessageDialog(MAIN_WIN,
										  "For help using LiveGraph " + LiveGraph.version + " please refer to the user manual at:\n" +
										  "http://www.live-graph.org/userManual.html\n\n" +
										  "In addition you can visit our support forums at:\n" +
										  "http://sourceforge.net/forum/?group_id=191061\n\n" +
										  "For further information browse the LiveGraph website at:\n" +
										  "http://www.live-graph.org\n ",
										  "Help",
										  JOptionPane.INFORMATION_MESSAGE);
		}
	});
	menu.add(mItem);
	mItem = new JMenuItem("Info...");
	mItem.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			JOptionPane.showMessageDialog(MAIN_WIN,
										  "LiveGraph\nVersion " + LiveGraph.version + "\n" +
										  "http://www.live-graph.org\n\n" +
										  "Developed by Greg Paperin\n(http://www.paperin.org)\n" +
										  "at Monash University\n ",
										  "LiveGraph " + LiveGraph.version + " credits",
										  JOptionPane.INFORMATION_MESSAGE);
		}
	});
	menu.add(mItem);
	menuBar.add(menu);
}

/**
 * Displays a log message.
 * @param s A message.
 */
private void logLn(String s) {
	messageArea.append(s);
	JScrollBar sb = ((JScrollPane) messageArea.getParent().getParent()).getVerticalScrollBar();
	if (null != sb)
		sb.setValue(sb.getMaximum());
	messageArea.append("\n");
}

/**
 * Displays an info message.
 * @param s A message
 */
public void logInfoLn(String s) {
	StringBuffer b = new StringBuffer();
	b.append("[INFO](");
	b.append(logTimestampFormat.format(new Date()));
	b.append("): ");
	b.append(s);
	logLn(b.toString());
}

/**
 * Displays an error message.
 * @param s A message
 */
public void logErrorLn(String s) {
	StringBuffer b = new StringBuffer();
	b.append("[ERR ](");
	b.append(logTimestampFormat.format(new Date()));
	b.append("): ");
	b.append(s);
	logLn(b.toString());
}

/**
 * Displays an success message.
 * @param s A message
 */
public void logSuccessLn(String s) {
	StringBuffer b = new StringBuffer();
	b.append("[SUCC](");
	b.append(logTimestampFormat.format(new Date()));
	b.append("): ");
	b.append(s);
	logLn(b.toString());
}

/**
 * Update the menu state of {@link #dataFileSettingsDisplayMenuItem}.
 * @param state New state.
 */
public void fileSettingsDisplayStateChanged(boolean state) {
	dataFileSettingsDisplayMenuItem.setState(state);
}

/**
 * Update the menu state of {@link #graphSettingsDisplayMenuItem}.
 * @param state New state.
 */
public void graphSettingsDisplayStateChanged(boolean state) {
	graphSettingsDisplayMenuItem.setState(state);
}

/**
 * Update the menu state of {@link #seriesSettingsDisplayMenuItem}.
 * @param state New state.
 */
public void seriesSettingsDisplayStateChanged(boolean state) {
	seriesSettingsDisplayMenuItem.setState(state);
}

/**
 * Update the menu state of {@link #plotDisplayMenuItem}.
 * @param state New state.
 */
public void plotDisplayStateChanged(boolean state) {
	plotDisplayMenuItem.setState(state);
}

}
