package org.LiveGraph.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.FontMetrics;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import javax.swing.AbstractCellEditor;
import javax.swing.BorderFactory;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JColorChooser;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.ListSelectionModel;
import javax.swing.WindowConstants;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;

import org.LiveGraph.LiveGraph;
import org.LiveGraph.dataCache.CacheObserver;
import org.LiveGraph.dataCache.DataCache;
import org.LiveGraph.settings.DataSeriesSettings;
import org.LiveGraph.settings.GraphSettings;
import org.LiveGraph.settings.ObservableSettings;
import org.LiveGraph.settings.SettingsObserver;
import org.LiveGraph.settings.DataSeriesSettings.TransformMode;

import com.softnetConsult.utils.collections.ReadOnlyIterator;
import com.softnetConsult.utils.swing.DisEnablingPanel;
import com.softnetConsult.utils.swing.SwingTools;


/**
 * The "Series Settings" window of the application.
 * 
 * <p style="font-size:smaller;">This product includes software developed by the
 *    <strong>LiveGraph</strong> project and its contributors.<br />
 *    (<a href="http://www.live-graph.org" target="_blank">http://www.live-graph.org</a>)<br />
 *    Copyright (c) 2007 G. Paperin.<br />
 *    All rights reserved.
 * </p>
 * <p style="font-size:smaller;">File: SeriesSettingsWindow.java</p> 
 * <p style="font-size:smaller;">Redistribution and use in source and binary forms, with or
 *    without modification, are permitted provided that the following terms and conditions are met:
 * </p>
 * <p style="font-size:smaller;">1. Redistributions of source code must retain the above
 *    acknowledgement of the LiveGraph project and its web-site, the above copyright notice,
 *    this list of conditions and the following disclaimer.<br />
 *    2. Redistributions in binary form must reproduce the above acknowledgement of the
 *    LiveGraph project and its web-site, the above copyright notice, this list of conditions
 *    and the following disclaimer in the documentation and/or other materials provided with
 *    the distribution.<br />
 *    3. All advertising materials mentioning features or use of this software or any derived
 *    software must display the following acknowledgement:<br />
 *    <em>This product includes software developed by the LiveGraph project and its
 *    contributors.<br />(http://www.live-graph.org)</em><br />
 *    4. All advertising materials distributed in form of HTML pages or any other technology
 *    permitting active hyper-links that mention features or use of this software or any
 *    derived software must display the acknowledgment specified in condition 3 of this
 *    agreement, and in addition, include a visible and working hyper-link to the LiveGraph
 *    homepage (http://www.live-graph.org).
 * </p>
 * <p style="font-size:smaller;">THIS SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY
 *    OF ANY KIND, EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 *    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND  NONINFRINGEMENT. IN NO EVENT SHALL
 *    THE AUTHORS, CONTRIBUTORS OR COPYRIGHT  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 *    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING  FROM, OUT OF OR
 *    IN CONNECTION WITH THE SOFTWARE OR THE USE OR  OTHER DEALINGS IN THE SOFTWARE.
 * </p>
 * 
 * @author Greg Paperin (<a href="http://www.paperin.org" target="_blank">http://www.paperin.org</a>)
 * @version {@value org.LiveGraph.LiveGraph#version}
 */
public class SeriesSettingsWindow extends JFrame implements CacheObserver, SettingsObserver,
																	SeriesHighlightListener {

private static final long HIGHLIGHT_LEN = 1500; 

private String[] seriesLabels = null; 
private AbstractTableModel tableModel = null;
private JTable table = null;
private ListSelectionModel selectionModel = null;

private JComboBox scaleTypeCombo = null;

private DisEnablingPanel topPanel;
private JButton advPanelButt = null, advGoButt = null;
private JPanel advPanel = null;
private int helpCols = 0;
private JTextField advFrom = null, advTo = null, advEvery = null;
private JComboBox advAction = null;

/**
 * This is the default constructor.
 */
public SeriesSettingsWindow() {
	super();
	initialize();
}

/**
 * This method initializes the window.
 */
private void initialize() {
	
	// Window settings:
	
	//final SeriesSettingsWindow SERIESSETTINGS_WIN = this;
	this.setDefaultCloseOperation(WindowConstants.DO_NOTHING_ON_CLOSE);
	Dimension frameDim = new Dimension(450, 200);
	this.setPreferredSize(frameDim);
	this.setBounds(480, 510, frameDim.width, frameDim.height);	
	this.setTitle("Data series settings (LiveGraph)");
	
	// Hide-show listener:
	
	this.addWindowListener(new WindowAdapter() {
		@Override public void windowClosing(WindowEvent e) {
			LiveGraph.application().setDisplaySeriesSettingsWindow(false);
		}
	});
	
	// Layout:
	
	getContentPane().setLayout(new BorderLayout(0, 0));
	
	// Buttons at the top:
	
	JPanel panel = null;
	JButton button = null;
	JLabel label = null;
	
	topPanel = new DisEnablingPanel(new BorderLayout());
	getContentPane().add(topPanel, BorderLayout.NORTH);
	
	panel = new JPanel(new FlowLayout(FlowLayout.LEFT, 1, 1));
	topPanel.add(panel, BorderLayout.NORTH);
	
	button = new JButton("Show all");
	button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (null == tableModel)  return;
			DataSeriesSettings dss = LiveGraph.application().getDataSeriesSettings();
			dss.setShowAll(0, tableModel.getRowCount() - 1, true);			
			tableModel.fireTableDataChanged();
        }
	});
	panel.add(button);
	
	button = new JButton("Hide all");
	button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (null == tableModel)  return;
			DataSeriesSettings dss = LiveGraph.application().getDataSeriesSettings();
			dss.setShowAll(0, tableModel.getRowCount() - 1, false);			
			tableModel.fireTableDataChanged();
        }
	});
	panel.add(button);
	
	button = new JButton("Toggle all");
	button.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			if (null == tableModel)  return;
			DataSeriesSettings dss = LiveGraph.application().getDataSeriesSettings();
			dss.setShowToggleAll(0, tableModel.getRowCount() - 1);			
			tableModel.fireTableDataChanged();
        }
	});
	panel.add(button);
	
	// Advanced selection panel:
	advPanel = new JPanel(new FlowLayout(FlowLayout.LEFT, 1, 1));
	
	advPanelButt = new JButton(">>");
	advPanelButt.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			setAdvancedPanelVisibility(!advPanel.isVisible());			
        }
	});
	panel.add(advPanelButt);
		
	topPanel.add(advPanel, BorderLayout.CENTER);	
	
	advPanel.add(label = new JLabel("From:"));
	label.setFont(SwingTools.getPlainFont(label));
	advFrom = new JTextField("0", 4);
	advPanel.add(advFrom);
	
	advPanel.add(label = new JLabel("To:"));
	label.setFont(SwingTools.getPlainFont(label));
	advTo = new JTextField("10000", 4);
	advPanel.add(advTo);
	
	advPanel.add(label = new JLabel("Every:"));
	label.setFont(SwingTools.getPlainFont(label));
	advEvery = new JTextField("10", 8);
	advPanel.add(advEvery);
	
	advAction = new JComboBox(new String[] {"Show", "Hide", "Toggle"});
	advAction.setFont(SwingTools.getPlainFont(advAction));
	advPanel.add(advAction);
	
	advGoButt = new JButton("Go");
	advGoButt.addActionListener(new ActionListener() {
		public void actionPerformed(ActionEvent e) {
			runAdvancedSelector();			
        }
	});
	advPanel.add(advGoButt);
	
	// Main table:
	
	panel = new JPanel(new BorderLayout(0, 0));
	
	table = createTable();
	panel.add(new JScrollPane(table), BorderLayout.CENTER);
	getContentPane().add(panel, BorderLayout.CENTER);
	
	setAdvancedPanelVisibility(false);
	
} // private void initialize()

/**
 * Creates and initialises the labels table.
 * @return The labels table.
 */
private JTable createTable() {
	
	final String[] scaleModes = new String[] {"Actual value", "Transformed into [0..1]",
											  "Scaled by specified value"};
	final String[] colNames = new String[] {"Show", "Label", "Colour", "Transformation", "Scale factor"};
	final String[] helperColumnNames = new String[] {"Index"};
	
	seriesLabels = new String[0];
	
	tableModel = new AbstractTableModel() {
				
		public int getColumnCount() {
	        return colNames.length + helpCols;
        }

		public int getRowCount() {
	        return seriesLabels.length;
        }
		
		@Override
		public String getColumnName(int col) {
			if (col < helpCols)
				return helperColumnNames[col];
			return colNames[col - helpCols];
		}

		public Object getValueAt(int row, int col) {
			if (1 == helpCols && 0 == col)
				return row;			
			switch (col - helpCols) {
				case 0: return LiveGraph.application().getDataSeriesSettings().getShow(row);
				case 1: return seriesLabels[row];
				case 2: return LiveGraph.application().getDataSeriesSettings().getColour(row);
				case 3: switch(LiveGraph.application().getDataSeriesSettings().getTransformMode(row)) {
							case Transform_None:   return scaleModes[0];
							case Transform_In0to1: return scaleModes[1];
							case Transform_SetVal: return scaleModes[2];
							default: throw new Error("This cannot happen!");
						}
				case 4: return LiveGraph.application().getDataSeriesSettings().getScaleFactor(row);
				default: throw new Error("Forgot to provide getValueAt for table column " + col + ".");
			}
        }
		
		@Override
		public void setValueAt(Object val, int row, int col) {
			super.setValueAt(val, row, col);			
			switch (col - helpCols) {
				case 0: LiveGraph.application().getDataSeriesSettings().setShow(row, ((Boolean) val).booleanValue());
						break;
			  //case 1: same as the default case.
				case 2: LiveGraph.application().getDataSeriesSettings().setColour(row, (Color) val);
						break;
				case 3: if (scaleModes[0].equals(val))
							LiveGraph.application().getDataSeriesSettings().setTransformMode(row, TransformMode.Transform_None);
						else if (scaleModes[1].equals(val))
							LiveGraph.application().getDataSeriesSettings().setTransformMode(row, TransformMode.Transform_In0to1);
						else if (scaleModes[2].equals(val))
							LiveGraph.application().getDataSeriesSettings().setTransformMode(row, TransformMode.Transform_SetVal);
						else
							throw new Error("Unexpected scale mode (" + val + ")!");
						break;
				case 4: LiveGraph.application().getDataSeriesSettings().setScaleFactor(row, ((Double) val).doubleValue());
						break;
				default: throw new Error("Column " + col + " is not supposed to be editable.");
			}
		}
		
		@Override
		public boolean isCellEditable(int row, int col) {
			return (col >= helpCols && col != 1 + helpCols);			
		}
		
		@Override
		public Class<?> getColumnClass(int col) {
			if (1 == helpCols && 0 == col)
				return Integer.class;
			switch (col - helpCols) {
				case 0: return Boolean.class;
				case 1: return String.class;
				case 2: return Color.class;
				case 3: return String.class;
				case 4: return Double.class;
				default: throw new Error("Forgot to provide getColumnClass for table column " + col + ".");
			}
	    }
		
	}; // AbstractTableModel model = new AbstractTableModel()
	
	
	//JTable table = new JTable(tableModel);
	
	JTable table = new JTable(tableModel) {
		@Override public void changeSelection(int rowIndex, int columnIndex, boolean toggle, boolean extend) {			
			; // Prevent the table selection to be changed by user input.
		}
	};
	
	
	selectionModel = table.getSelectionModel();
	selectionModel.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
	
	scaleTypeCombo = new JComboBox(scaleModes);		
	scaleTypeCombo.setFont(SwingTools.getPlainFont(scaleTypeCombo));
	
	table.getColumnModel().getColumn(3 + helpCols).setCellEditor(new DefaultCellEditor(scaleTypeCombo));
	
	table.setDefaultEditor(Color.class, new ColourEditor());
	table.setDefaultRenderer(Color.class, new TableCellRenderer() {
		private JLabel label = new JLabel();
		public Component getTableCellRendererComponent(JTable table, Object colour,
													   boolean isSelected, boolean hasFocus, int row, int column) {			
			label.setOpaque(true);
			Color brdCol = isSelected ? table.getSelectionBackground() : table.getBackground();
			label.setBorder(BorderFactory.createLineBorder(brdCol, 3));
			label.setBackground((Color) colour);
			return label;
		}
	
	});
	
	Component rc = table.getTableHeader().getDefaultRenderer().getTableCellRendererComponent(table, null,
																							 false, false, 0, 0);
	FontMetrics fm = rc.getFontMetrics(rc.getFont());
	int w, sumW;
	
	sumW = w = fm.stringWidth(colNames[0 + helpCols]) + 10;
	table.getColumnModel().getColumn(0 + helpCols).setPreferredWidth(w);
	
	sumW += w = fm.stringWidth(colNames[2 + helpCols]) + 10;
	table.getColumnModel().getColumn(2 + helpCols).setPreferredWidth(w);
	
	w = 0;
	for (String sm : scaleModes) { w = Math.max(w, fm.stringWidth(sm)); }
	sumW += w = Math.max(w, fm.stringWidth(colNames[3 + helpCols])) + 10;	
	table.getColumnModel().getColumn(3 + helpCols).setPreferredWidth(w);
	
	sumW += w = fm.stringWidth(colNames[4 + helpCols]) + 10;
	table.getColumnModel().getColumn(4 + helpCols).setPreferredWidth(w);
	
	w = table.getPreferredScrollableViewportSize().width - sumW - 20;
	table.getColumnModel().getColumn(1 + helpCols).setPreferredWidth(w);
	
	return table;
	
} // private JTable createTable()


/**
 * Shows and hides the advanced selector panel.
 * @param show Whether to show the advanced selector panel.
 */
private void setAdvancedPanelVisibility(boolean show) {
	
	advPanelButt.setText(show ? "<<" : ">>");
	advPanel.setVisible(show);
	
	if (show) {
		helpCols = 1;
	} else {
		helpCols = 0;
	}
		
	tableModel.fireTableStructureChanged();
	table.getColumnModel().getColumn(3 + helpCols).setCellEditor(new DefaultCellEditor(scaleTypeCombo));
}

/**
 * Executes the advanced selection of data series.
 */
private void runAdvancedSelector() {
	
	// Get values:
	int start, end;
	List<Integer> mods;
	int action;
	try {		
		start = Integer.parseInt(advFrom.getText());
		end = Integer.parseInt(advTo.getText());		
		
		StringTokenizer tok = new StringTokenizer(advEvery.getText(), ",; ");
		mods = new ArrayList<Integer>();
		while(tok.hasMoreTokens()) {
			int t = Integer.parseInt(tok.nextToken());
			if (t > 0)
				mods.add(t);
		}
		
		Object act = advAction.getSelectedItem();
		if ("Show".equals(act))
			action = 1;
		else if ("Hide".equals(act))
			action = 2;
		else if ("Toggle".equals(act))
			action = 3;
		else
			action = -1;
		
	} catch(NumberFormatException e) {
		JOptionPane.showMessageDialog(this, e.getMessage(), "Invalid number format", JOptionPane.ERROR_MESSAGE);
		return;
	}
	
	if (0 > start)
		start = 0;
	if (tableModel.getRowCount() <= start)
		start = tableModel.getRowCount() - 1;
	
	if (0 > end || tableModel.getRowCount() <= end)
		end = tableModel.getRowCount() - 1;
	
	if (start > end) {
		int t = start; start = end; end = t;
	}
	
	advFrom.setText(Integer.toString(start));
	advTo.setText(Integer.toString(end));
	
	topPanel.setEnabled(false);
	DataSeriesSettings setts = LiveGraph.application().getDataSeriesSettings();	
	for (int r = start; r <= end; r++) {
		
		advGoButt.setText(Integer.toString(r));
		
		int offs = r - start;
		boolean apply = false;
		for (int m = 0; !apply && m < mods.size(); m++) {
			apply = (0 == offs % mods.get(m));
		}
		
		if (!apply)
			continue;
		
		switch(action) {
			case 1: setts.setShow(r, true); break;
			case 2: setts.setShow(r, false); break;
			case 3: setts.setShow(r, !setts.getShow(r)); break;
			default: throw new Error("Bug! This line should never be reached. Have fun debugging.");
		}
	}
	advGoButt.setText("Go");
	topPanel.setEnabled(true);	
}

/**
 * Locally updates the series-lables when they have been changed in the data cache.
 */
public void cacheEventFired(DataCache cache, CacheEvent event) {
	
	switch(event) {
		case UpdateLabels:
			seriesLabels = new String[cache.countDataSeries()];
			ReadOnlyIterator<String> labs = cache.iterateDataSeriesLabels();
			int i = 0;
			while (labs.hasNext())
				seriesLabels[i++] = labs.next();
			tableModel.fireTableDataChanged();
			break;
		case ChangeMode:
		case UpdateData:
		case UpdateDataFileInfo:		
			break;
		default:
			throw new Error("This case is impossible!");
			
	}
} // public void cacheEventFired(DataCache cache, CacheEvent event)


/**
 * Dispatches the settings change events.
 * @see #settingHasChanged(DataSeriesSettings, String)
 * @see #settingHasChanged(GraphSettings, String)
 */
public void settingHasChanged(ObservableSettings settings, Object info) {
	
	if (null == info || null == settings)
		return;
	
	if ((settings instanceof DataSeriesSettings) && (info instanceof String)) {
		settingHasChanged((DataSeriesSettings) settings, (String) info);
		return;
	}
	
	if ((settings instanceof GraphSettings) && (info instanceof String)) {
		settingHasChanged((GraphSettings) settings, (String) info);
		return;
	}
} // public void settingHasChanged(ObservableSettings settings, Object info)

/**
 * Updates the table display when series settings were loaded from a file.
 * @param settings Series settings.
 * @param info Event info.
 */
public void settingHasChanged(DataSeriesSettings settings, String info) {
	
	if (null == info || null == settings)
		return;
	
	if (info.equals("load")) {
		tableModel.fireTableDataChanged();
		return;
	}
	
	if (info.startsWith("Show.")) {
		int r = Integer.parseInt(info.substring(info.lastIndexOf('.') + 1));
		tableModel.fireTableRowsUpdated(r, r);
		return;
	}
} // public void settingHasChanged(DataSeriesSettings settings, String info)

/**
 * When a series was selected as x-axis, the corresponding setting is highlighted for a second.
 * @param settings Graph settings.
 * @param info Event info.
 */
public void settingHasChanged(GraphSettings settings, String info) {
	
	if (null == info || null == settings)
		return;
	
	if (info.equals("XAxisType") || info.equals("XAxisSeriesIndex")) {
		
		if (GraphSettings.XAxisType.XAxis_DSNum == settings.getXAxisType())
			return;
	
		final int serInd = settings.getXAxisSeriesIndex();
		tableModel.fireTableRowsUpdated(serInd, serInd);
		selectionModel.setSelectionInterval(serInd, serInd);
		(new Thread(new Runnable() {
			public void run() {
				try { Thread.sleep(HIGHLIGHT_LEN); } catch (InterruptedException e) {}
				if (!selectionModel.getValueIsAdjusting()
						&& selectionModel.getMinSelectionIndex() == serInd
						&& selectionModel.getMaxSelectionIndex() == serInd) {
					selectionModel.clearSelection();
				}
            }
		}, "SeriesMarkedAsXAxis-TableSelectionController")).start();
		return;		
	}
	
} // public void settingHasChanged(GraphSettings settings, String info)

public void highlightSeries(List<Integer> seriesIndices) {
	
	if (null == seriesIndices)
		return;
	
	selectionModel.clearSelection();
	
	if (seriesIndices.isEmpty())
		return;
	
	for (int s : seriesIndices)
		selectionModel.addSelectionInterval(s, s);
	/*
	(new Thread(new Runnable() {
		public void run() {
			try { Thread.sleep(HIGHLIGHT_LEN); } catch (InterruptedException e) {}
			selectionModel.clearSelection();
        }
	}, "HighlightSeries-TableSelectionController")).start();
	*/
	
} // public void highlightSeries(List<Integer> seriesIndices)

/**
 * A colour selection cell editor for the settings table.  
 */
private class ColourEditor extends AbstractCellEditor implements TableCellEditor {

	private Color selColor;
	private JButton button;
	
	public ColourEditor() {
		button = new JButton();
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				button.setBackground(selColor);
				Color c = JColorChooser.showDialog(button, "Choose a colour for the data series.", selColor);
				if (null != c) 
					selColor = c;
				fireEditingStopped();
            }
		});	
	}
	
	public Object getCellEditorValue() {
		return selColor;
	}

	public Component getTableCellEditorComponent(JTable table, Object value,
												 boolean isSelected, int row, int column) {
		selColor = (Color) value;
		Color brdCol = isSelected ? table.getSelectionBackground() : table.getBackground();
		button.setBorder(BorderFactory.createLineBorder(brdCol, 3));
		return button;
	}
} //private class ColourEditor

} // public class SeriesSettingsWindow  
